@@ -804,12 +804,14 @@ domain items or items specified as *@domain. The value of the second argument
 must then be given as 0.
 
 Domains are always lower cased before the match. Local parts are also lower
-cased unless an item of the form +caseful is encountered. The casefulness of
-other matching (where possible) is also controlled by this.
+cased if "caseless" is set, until an item of the form +caseful is encountered
+in the list. The casefulness of other matching (where possible) is also
+controlled by this.
 
 Arguments:
   origaddress   address to test
   domain_offset offset to the domain in the address
+  caseless      TRUE if caseless lower part matching initially
   list          string list to check against
   expand_setup  controls setting up of $n variables - passed through
                 to match_check_string (q.v.)
@@ -824,11 +826,10 @@ Returns:        TRUE for a positive match, or end list after a negation;
 */
 
 BOOL
-match_address_list(char *origaddress, int domain_offset, char **listptr,
-  int expand_setup, int *separator, BOOL ldr)
+match_address_list(char *origaddress, int domain_offset, BOOL caseless,
+  char **listptr, int expand_setup, int *separator, BOOL ldr)
 {
 int llen;
-BOOL caseless = TRUE;
 BOOL endyield = FALSE;
 char *ss, *p;
 char *list = *listptr;
@@ -861,21 +862,24 @@ if (list == NULL)
   return FALSE;
   }
 
-/* Ensure the domain is lower-cased before doing any comparisons. The mail RFCs
-specify that local parts are caseful things, and Exim always preserves the case
-for remote local parts. However, RFC 2505 recommends that for spam checking,
-local parts should be caselessly compared. Therefore, Exim now forces the local
-part into lower case here, but if it encounters a magic item "+caseful" in the
-list, it restores a caseful copy from the original address. */
-
-strcpy(address, origaddress);
-for (p = address; *p != 0; p++) *p = tolower(*p);
-
 /* Compute the length of the local part; if domain_offset == 0 the length is
 negative. */
 
 llen = domain_offset - 1;
 
+/* If caseless is set, ensure the domain is lower-cased before doing any
+comparisons. The mail RFCs specify that local parts are caseful things, and
+Exim always preserves the case for remote local parts. However, RFC 2505
+recommends that for spam checking, local parts should be caselessly compared.
+Therefore, Exim now forces the local part into lower case here for most calls
+to this function (caseless == TRUE), but if it encounters a magic item
+"+caseful" in the list, it restores a caseful copy from the original address.
+*/
+
+strcpy(address, origaddress);
+for (p = address + ((caseless || llen < 0)? 0 : llen); *p != 0; p++)
+  *p = tolower(*p);
+
 /* If expand_setup is zero, we need to set up $0 to the whole thing, in
 case there is a match. Can't use the built-in facilities of match_check_string,
 as we may just be calling that for part of the address (the domain). */
@@ -1050,6 +1054,7 @@ if (s == NULL) return FAIL;
 yield = match_address_list(
   address,
   (address[0] == 0)? 0 : strchr(address, '@') - address + 1,
+  TRUE,
   &s,
   FALSE,
   &sep,
@@ -1220,8 +1225,8 @@ for (;;)
     subjbuffer[len-1] = 0;
     s = strchr(subjbuffer, '@');
     domain_offset = (s == NULL)? 0 : s - subjbuffer + 1;
-    if (match_address_list(subjbuffer, domain_offset, &list, -1, &sep, FALSE))
-      printf("OK\n");
+    if (match_address_list(subjbuffer, domain_offset, TRUE, &list, -1, &sep,
+      FALSE)) printf("OK\n");
     else
       printf("Failed%s\n", search_find_defer? " (lookup deferred)" : "");
     }
