@@ -760,9 +760,9 @@ while ((ent = readdir(dir)) != NULL)
     continue;
     }
 
-  if ((statbuf.st_mode & S_IFREG) != 0)
+  if ((statbuf.st_mode & S_IFMT) == S_IFREG)
     sum += statbuf.st_size;
-  else if ((statbuf.st_mode & S_IFDIR) != 0)
+  else if ((statbuf.st_mode & S_IFMT) == S_IFDIR)
     sum += check_dir_size(buffer, &count, regex);
   }
 
@@ -1092,10 +1092,9 @@ int yield = FAIL;
 int i;
 
 #ifdef SUPPORT_MBX
-int temp_fd = -1;
+FILE *temp_file = NULL;
 int save_fd = 0;
 int mbx_lockfd = -1;
-char *tempname = NULL;
 char mbx_lockname[40];
 #endif
 
@@ -2415,7 +2414,10 @@ if (ob->quota_warn_threshold_value > 0)
   {
   int threshold = ob->quota_warn_threshold_value;
   if (ob->quota_warn_threshold_is_percent)
-    threshold = (ob->quota_value * threshold) / 100;
+    threshold = (int)(((double)ob->quota_value * threshold) / 100);
+  DEBUG(2)
+    debug_printf("quota threshold = %d old size = %d message size = %d\n",
+      threshold, saved_size, message_size);
   if (saved_size <= threshold && saved_size + message_size > threshold)
     addr->special_action = SPECIAL_WARN;
   }
@@ -2424,26 +2426,22 @@ if (ob->quota_warn_threshold_value > 0)
 to a temporary file, and then copy it to the real file once we know its size.
 This is the most straightforward way of getting the correct length in the
 separator line. So, what we do here is to save the real file descriptor, and
-replace it with one for a temporary file. The temporary file gets unlinked once
-opened, so that it goes away on closure. */
+replace it with one for a temporary file. */
 
 #ifdef SUPPORT_MBX
 if (yield == OK && ob->mbx_format)
   {
-  tempname = tmpnam(NULL);
-  if ((temp_fd = open(tempname, (O_RDWR|O_CREAT|O_TRUNC), 0600)) < 0 ||
-      unlink(tempname) < 0)
+  temp_file = tmpfile();
+  if (temp_file == NULL)
     {
     addr->basic_errno = errno;
-    addr->message = string_sprintf("while setting up temporary file %s",
-      tempname);
-    if (temp_fd >= 0) close(temp_fd);
+    addr->message = "while setting up temporary file";
     yield = DEFER;
     goto RETURN;
     }
   save_fd = fd;
-  fd = temp_fd;
-  DEBUG(9) debug_printf("writing to temporary file %s\n", tempname);
+  fd = fileno(temp_file);
+  DEBUG(9) debug_printf("writing to temporary file\n");
   }
 #endif
 
@@ -2460,9 +2458,7 @@ if (tblock->local_smtp != local_smtp_off && yield == OK)
       !transport_write_string(fd, "HELO %s%s\n", primary_hostname, cr))
     yield = DEFER;
   else
-    if (!transport_write_string(fd, "MAIL FROM:<%s>%s\n",
-         ((addr->errors_address != NULL)?
-           addr->errors_address : sender_address), cr))
+    if (!transport_write_string(fd, "MAIL FROM:<%s>%s\n", return_path, cr))
       yield = DEFER;
   else
     {
@@ -2536,7 +2532,7 @@ message in MBX format into the real file. Otherwise use the temporary name in
 any messages. */
 
 #ifdef SUPPORT_MBX
-if (temp_fd >= 0 && ob->mbx_format)
+if (temp_file != NULL && ob->mbx_format)
   {
   int save_errno;
 
@@ -2545,15 +2541,14 @@ if (temp_fd >= 0 && ob->mbx_format)
   if (yield == OK)
     {
     transport_count = 0;   /* Reset transport count for actual write */
-    yield = copy_mbx_message(fd, temp_fd, saved_size);
+    yield = copy_mbx_message(fd, fileno(temp_file), saved_size);
     }
-  else if (errno >= 0) dataname = tempname;
+  else if (errno >= 0) dataname = "temporary file";
 
-  /* Preserve errno while closing the temporary file, which will lose it,
-  since it was unlinked above. */
+  /* Preserve errno while closing the temporary file. */
 
   save_errno = errno;
-  close(temp_fd);
+  fclose(temp_file);
   errno = save_errno;
   }
 #endif
